@@ -24,13 +24,14 @@ use constant {
     ENTITY_MASK  => 0xFFFFF, # Used to convert GUIDs to entity numbers
     VERSION_MASK => 0xFFF,   # Used to convert GUIDs to entity versions
     ENTITY_SHIFT => 20,      # The size of the entity number within a GUID
+    NULL_ENTITY  => 0,       # The null entity
 };
 
 ## Entity "methods"
 
 my $version = sub ($e) { $e >> ENTITY_SHIFT };
 my $entity  = sub ($e) { $e &  ENTITY_MASK  };
-my $is_null = sub ($e) { $e->$entity == ENTITY_MASK  };
+my $is_null = sub ($e) { $e->$entity == NULL_ENTITY };
 my $format  = sub ($e) { sprintf '%012b:%020b', $e->$version, $e->$entity };
 
 ## Private, hidden methods
@@ -40,8 +41,8 @@ my $add_version = sub ($self, $index) {
 };
 
 my $generate_guid = sub ($self) {
-    die 'Exceeded maximum number of entities'
-        if @{ $self->{entities} //= [] } >= ENTITY_MASK - 1;
+    Carp::croak 'Exceeded maximum number of entities'
+        if @{ $self->{entities} } >= ENTITY_MASK;
 
     my $guid = @{ $self->{entities} };
     push @{ $self->{entities} }, $guid;
@@ -52,7 +53,7 @@ my $generate_guid = sub ($self) {
 my $recycle_guid = sub ($self) {
     my $next = $self->{available};
 
-    die 'Cannot recycle GUID if none has been released'
+    Carp::croak 'Cannot recycle GUID if none has been released'
         if $next->$is_null;
 
     my $ver = $self->{entities}[$next]->$version;
@@ -80,15 +81,17 @@ my $get = sub ( $self, $unsafe, $guid, @types ) {
 ## Public methods
 
 sub new ( $class ) {
-    bless { available => ENTITY_MASK }, $class;
+    my $self = bless {}, $class;
+    $self->clear;
+    $self;
 }
 
-sub created ($self) { scalar @{ $self->{entities} // [] } }
+sub created ($self) { scalar @{ $self->{entities} } - 1 }
 
 # Get the number of created entities that are still valid; that is, that have
 # not been deleted.
 sub alive ($self) {
-    my $size = @{ $self->{entities} // [] };
+    my $size = @{ $self->{entities} } - 1;
     my $current = $self->{available};
 
     until ( $current->$is_null ) {
@@ -110,8 +113,8 @@ sub clear ($self) {
 
     # Parameters used for recycling entity GUIDs
     # See https://skypjack.github.io/2019-05-06-ecs-baf-part-3
-    delete $self->{entities};
-    $self->{available} = ENTITY_MASK;
+    $self->{entities} = [ undef ];
+    $self->{available} = NULL_ENTITY;
 
     return;
 }
@@ -275,7 +278,7 @@ sub view ( $self, @types ) {
         return Game::Entities::View->new(
             map {; $self->$add_version( $_->$entity ) => [] }
                 grep $self->valid( $_ ),
-                @{ $self->{entities} }
+                @{ $self->{entities} }[ 1 .. $#{ $self->{entities} } ]
         )
     }
 
@@ -367,6 +370,7 @@ delete $Game::Entities::{$_} for qw(
     DENSE
     ENTITY_MASK
     ENTITY_SHIFT
+    NULL_ENTITY
     SPARSE
     VERSION_MASK
 );
